# pip install colorama

import sys
import time
from pathlib import Path

from colorama import Fore, init
from transformers import AutoTokenizer, AutoModelForSeq2SeqLM

init()

tokenizer = AutoTokenizer.from_pretrained(Path.cwd() / 'model' / 'en-ru-local')
model = AutoModelForSeq2SeqLM.from_pretrained(Path.cwd() / 'model' / 'en-ru-local')


def translate_phrase(phrase: str) -> str:
    """
    Перевод фраз из файла, вывод перевода в терминал.
    Функция возвращает переведенную фразу.
    """
    inputs = tokenizer(phrase, return_tensors="pt")
    output = model.generate(**inputs, max_new_tokens=100)
    out_text = tokenizer.batch_decode(output, skip_special_tokens=True)
    print(f'\r{Fore.YELLOW}   {out_text[0]}', end="")
    return out_text[0]


def load_files(path: str):
    """
    Сканирование директории на наличие файлов ".srt".
    Итерация по списку найденных файлов. Открытие каждого файла и
    итерация по нему построчно. Проверка, является ли начало строки и конец строки
    цифрой. В это случае добавляем строку в результирующий список без изменений.
    Также и с пустыми строками. Добавляем в список с переносом строки.
    В других случаях отправляем фразу для перевода в функцию и добавляем в список
    возвращенную переведенную фразу.
    После окончания итерации по файлу передаем список в функцию для сохранения результатов.
    Очищаем список и итерируемся по следующему файлу.
    """
    translate_text = []
    files = [x for x in Path(path).iterdir() if Path(x).suffix in [".srt", ".vtt"]]
    for nm, file in enumerate(files):
        print(f"\n{Fore.GREEN}[{nm + 1}/{len(files)}] Обработка файла: {Path(file).name}")
        text = (x.strip() for x in open(file, "r", encoding="utf-8"))
        for txt in text:
            if not txt:
                translate_text.append(" \n")
            elif txt == "WEBVTT":
                translate_text.append(f'{txt}\n')
            elif txt[0].isdigit() and txt[-1].isdigit():
                translate_text.append(f'{txt}\n')
            else:
                translate_text.append(f'{translate_phrase(txt)}\n')
        save_srt(path, file, translate_text)
        translate_text.clear()


def save_srt(path: str, file: Path, translate_text: list):
    """
    Сохранение результатов перевода в файл субтитров.
    """
    with open(f'{Path(path) / Path(file).name.split(Path(file).suffix)[0]}.ru.srt', "w", encoding="utf-8") as fl:
        for phrase in translate_text:
            fl.write(phrase)


def main():
    path = input("Введите путь к папке с файлами для перевода\n>>>  ")
    time_start = time.monotonic()
    if not Path(path).exists() or not Path(path).is_dir():
        print("[!] Проверьте правильность ввода пути")
        sys.exit(0)
    load_files(path)
    print(f"\n\n{Fore.GREEN}Все файлы обработаны и сохранены")
    print(f'{Fore.CYAN}Затраченное время {Fore.RESET}| '
          f'{(int(time.monotonic() - time_start) // 3600) % 24:d} ч. '
          f'{(int(time.monotonic() - time_start) // 60) % 60:02d} м. '
          f'{int(time.monotonic() - time_start) % 60:02d} с.\n')


if __name__ == "__main__":
    main()
